exports.require = ['gardener.api','gardener.apps'];

var moment = require('moment');

exports.plugin = function(ops) {
	
	var haba = this;

	function wrapRemote(target) {
		if(typeof target == 'function') {
			return function() {
				var args = Array.apply(null, arguments);


				//listener
				args.push({
					error: function(err) {
						console.log(err.message || err);
					},
					status: function(message) {
						console.log(message);
					},
					end: function(err, result) {
						// console.log(!err ? 'Success'.green : 'Failed'.red);
					}
				});

				target.apply(null, args);
			}
		}

		if(typeof target == 'object') {
			for(var prop in target) {
				target[prop] = wrapRemote(target[prop]);
			}
		}
		

		return target;
	}

	ops.cli.onJs({
		gn: {
			getHistory: {
				desc: 'Returns app checkout history',
				exec: function(appName) {

					haba.plugin('gardener.apps').appHistory(appName, {
						status: function(){},
						end: function(err, result) {

							result.sort(function(a, b) {
								return a.createdAt > b.createdAt ? 1 : -1;
							})

							ops.cli.table(result, {
								columns: [{
									name: function(item) {
										return 'version ' + item.release
									}
								},
								{
									name: function(item) {
										return moment(item.createdAt).fromNow();
									}
								}]
							});
						}
					});
				}
			},
			
			status: {
				desc: 'Status of all applications',
				exec: function() {
					haba.plugin('gardener.garden').getStatus(function(status) {
						ops.cli.table(status, {
							columns: [{
								name: 'name',
								width: 10
							},
							{
								name: function(item) {
									var buffer = '',
									processes = item.processes.forEach(function(proc) {
										buffer += proc.node+': ' + proc.name + ' ' + proc.statename + ' ' + proc.description + '\n';
									});

									return buffer;
								}
							}]
						})
					});
				}
			}
		}
	})

	haba.onLoad('gardener.api', function(plugin) {
		ops.cli.onJs({ gn: wrapRemote(plugin.api) });
	});
}